"""
    Created on 2012-01-20
    @author: jldupont
"""
import logging, os, sys, json

from jldaws.tools_sys import retry
from jldaws.db.skv import SimpleKV

def run(domain_name=None, 
        category_name=None,
        trigger_topic=None 
        ,just_key=False
        ,just_key_value=False
        ,disable_pass_through=False
        ,**_
        ):

    db=SimpleKV()
    def connect():
        db.connect()
    
    logging.info("Connecting to SDB... CTRL-C to abort")
    retry(connect)
    
    def create():
        db.set_domain(domain_name)
        
    logging.info("Creating domain: %s... CTRL-C to abort" % domain_name)
    retry(create)
    
    # MAIN LOOP
    ###########
    ppid=os.getppid()
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid:  %s" % ppid)
    logging.info("Starting loop...")
    while True:
        if os.getppid()!=ppid:
            logging.warning("Parent terminated... exiting")
            break

        try:
            line=sys.stdin.readline().strip()
            if not disable_pass_through:
                sys.stdout.write(line+"\n")            
        except:
            raise Exception("broken stdin/stdout...")
            
        
        if trigger_topic is not None:
            if len(line)==0 or line=="":   ## safe than sorry
                continue
            
            try:
                jo=json.loads(line)
            except:
                logging.warning("Can't decode JSON from: %s" % line)
                continue
            
            try:
                topic=jo["topic"]
            except:
                logging.warning("No 'topic' field found: %s" % line)
                continue
            
            if topic!=trigger_topic:
                logging.debug("No specified topic: %s" % topic)
                continue
            
        entries=[]
        next_token=None
        while True:
            try:
                logging.debug("Getting batch...")
                batch, next_token=db.get_by_category(category=category_name, next_token=next_token)
            except Exception, e:
                logging.warning(e)
                batch=None
                break
            
            if batch is None:
                break     
                   
            entries.append(batch)
            
            if next_token is None:
                break
            
        for entry in entries:
            
            if just_key:
                try:    print entry["key"]
                except:
                    logging.debug("Entry without a 'key' field...")
                    
            if just_key_value:
                try:    print "%s\t%s" % (entry["key"], entry["value"])
                except:
                    logging.debug("Entry without a 'key'/'value' field(s)...")
             
            if not just_key and not just_key_value:       
                print entry
                
            
            
        



import os, sys, argparse
op=os.path

try:
    import jldaws
except:
    ### must be in dev mode then    
    this_dir=op.dirname(__file__)
    lib_path=op.abspath(op.join(this_dir, "..", ".."))
    sys.path.insert(0, lib_path)
    import jldaws




"""
    Created on 2012-01-20
    @author: jldupont
"""
import logging, os, sys, json
from time import sleep

from jldaws.tools_sys import retry
from jldaws.db.skv import SimpleKV

def run(domain_name=None 
        ,category_name=None
        ,default_value=None
        ,**_
        ):

    db=SimpleKV()
    def connect():
        db.connect()
    
    logging.info("Connecting to SDB... CTRL-C to abort")
    retry(connect)
    
    def create():
        db.set_domain(domain_name)
        
    logging.info("Creating domain: %s... CTRL-C to abort" % domain_name)
    retry(create)
    
    # MAIN LOOP
    ###########
    ppid=os.getppid()
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid:  %s" % ppid)
    logging.info("Starting loop...")
    while True:
        if os.getppid()!=ppid:
            logging.warning("Parent terminated... exiting")
            break

        try:
            line=sys.stdin.readline().strip()
        except:
            raise Exception("broken stdin...")
            
        if len(line)==0 or line=="":   ## safe than sorry
            logging.debug("Empty input line...")
            continue
        
        try:
            ino=json.loads(line)
            ino["category"]=ino.get("category", category_name)
            ino["value"]=ino.get("value", default_value)
            if ino.get("key", None) is None:
                logging.warning("Missing 'key' field on line: %s" % line)
                continue
            logging.debug("Input JSON: %s" % ino)
        except:
            ino={}
            try:
                bits=line.split("\t")
                if len(bits)>3 or len(bits)<1:
                    logging.warning("Invalid text input: %s" % line)
                    continue
                if len(bits)==3:
                    ino["category"]=bits[0]
                    ino["key"]=bits[1]
                    ino["value"]=bits[2]
                if len(bits)==2:
                    ino["category"]=category_name
                    ino["key"]=bits[0]
                    ino["value"]=bits[1]
                if len(bits)==1:
                    ino["category"]=category_name
                    ino["key"]=bits[0]
                    ino["value"]=default_value
                logging.debug("Input TEXT: %s" % ino)
            except:
                logging.warning("Error processing input: %s" % line)
                continue
        
        try:
            db.insert(ino["key"], ino["value"], category=ino["category"])
            logging.info("Progress: inserted 1 record")
        except:
            try:
                ### don't give up too easily
                sleep(1)
                db.insert(ino["key"], ino["value"], category=ino["category"])
                logging.info("Progress: inserted 1 record (after 2nd try)")
            except Exception,e:
                logging.debug(e)
                logging.warning("Can't insert in SDB...")
        
                    
                    
            
        
        